package vn.lecle.lecle_system_shortcuts

import android.annotation.TargetApi
import android.app.Activity
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothManager
import android.content.Context
import android.content.Intent
import android.content.pm.ActivityInfo
import android.media.AudioManager
import android.net.wifi.WifiManager
import android.os.Build
import android.provider.Settings
import androidx.annotation.NonNull
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat.startActivityForResult
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** LecleSystemShortcutsPlugin */
class LecleSystemShortcutsPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var activity: Activity
    private lateinit var context: Context

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "lecle_system_shortcuts")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
    }

    @RequiresApi(Build.VERSION_CODES.JELLY_BEAN_MR2)
    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "home" -> home()
            "back" -> back()
            "volDown" -> {
                val i: Int? = call.argument<Int>("streamType")
                val showVolumeUiFlag: Int? = call.argument<Int>("showVolumeUiFlag")
                volDown(i ?: 0, showVolumeUiFlag)
            }
            "volUp" -> {
                val i: Int? = call.argument<Int>("streamType")
                val showVolumeUiFlag: Int? = call.argument<Int>("showVolumeUiFlag")
                volUp(i ?: 0, showVolumeUiFlag)
            }
            "orientLandscape" -> orientLandscape()
            "orientPortrait" -> orientPortrait()
            "wifi" -> wifi()
            "checkWifi" -> result.success(checkWifi())
            "bluetooth" -> bluetooth()
            "checkBluetooth" -> result.success(checkBluetooth())
            else -> result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
    }

    private fun home() {
        activity.startActivity(Intent(Intent.ACTION_MAIN).addCategory(Intent.CATEGORY_HOME))
    }

    @TargetApi(Build.VERSION_CODES.ECLAIR)
    private fun back() {
        activity.onBackPressed()
    }

    private fun volDown(i: Int, showVolumeUiFlag: Int?) {
        val audioManager = activity.applicationContext.getSystemService(Context.AUDIO_SERVICE) as AudioManager
        if (showVolumeUiFlag != null) {
            audioManager.setStreamVolume(i, AudioManager.ADJUST_LOWER, showVolumeUiFlag)
        }

        audioManager.adjustVolume(AudioManager.ADJUST_LOWER, AudioManager.FLAG_PLAY_SOUND)
    }

    private fun volUp(i: Int, showVolumeUiFlag: Int?) {
        val audioManager = activity.applicationContext.getSystemService(Context.AUDIO_SERVICE) as AudioManager
        if (showVolumeUiFlag != null) {
            audioManager.setStreamVolume(i, AudioManager.ADJUST_RAISE, showVolumeUiFlag)
        }

        audioManager.adjustVolume(AudioManager.ADJUST_RAISE, AudioManager.FLAG_PLAY_SOUND)
    }

    private fun orientLandscape() {
        activity.requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE
    }

    private fun orientPortrait() {
        activity.requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED
    }

    private fun wifi() {
        if (Build.VERSION.SDK_INT < 29) {
            val wifiManager = activity.applicationContext.getSystemService(Context.WIFI_SERVICE) as WifiManager
            wifiManager.isWifiEnabled = !wifiManager.isWifiEnabled
        } else {
            val panelIntent = Intent(Settings.Panel.ACTION_WIFI)
            startActivityForResult(activity, panelIntent, 1, null)
        }
    }

    private fun checkWifi(): Boolean {
        val wifiManager = activity.applicationContext.getSystemService(Context.WIFI_SERVICE) as WifiManager
        return wifiManager.isWifiEnabled
    }

    @RequiresApi(Build.VERSION_CODES.JELLY_BEAN_MR2)
    @TargetApi(Build.VERSION_CODES.ECLAIR)
    private fun bluetooth() {
        val mBluetoothAdapter: BluetoothAdapter = if (Build.VERSION.SDK_INT < 31) {
            BluetoothAdapter.getDefaultAdapter()
        } else {
            val bluetoothManager: BluetoothManager = context.getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager
            bluetoothManager.adapter
        }

        if (mBluetoothAdapter.isEnabled) {
            mBluetoothAdapter.disable()
        } else {
            mBluetoothAdapter.enable()
        }
    }

    @TargetApi(Build.VERSION_CODES.ECLAIR)
    private fun checkBluetooth(): Boolean {
        val mBluetoothAdapter: BluetoothAdapter = if (Build.VERSION.SDK_INT < 31) {
            BluetoothAdapter.getDefaultAdapter()
        } else {
            val bluetoothManager: BluetoothManager = context.getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager
            bluetoothManager.adapter
        }

        return mBluetoothAdapter.isEnabled
    }
}
