import Flutter
import UIKit
import MediaPlayer
import Foundation
import Swift
import SystemConfiguration
import CoreBluetooth
import SwiftUI

public class SwiftLecleSystemShortcutsPlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "lecle_system_shortcuts", binaryMessenger: registrar.messenger())
        let instance = SwiftLecleSystemShortcutsPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "home":
            UIControl().sendAction(#selector(URLSessionTask.suspend), to: UIApplication.shared, for: nil)
            result("Sent to Home Screen")
        case "back":
            // Not working, use Navigator.pop(context) instead
            _ =  UIApplication.shared.keyWindow?.rootViewController?.navigationController?.popViewController(animated: true)
            result("Back to previous screen")
        case "vol":
            let val = AVAudioSession.sharedInstance().outputVolume
            result(val)
        case "setVol":
            var  _arguments = call.arguments as! [String : Any];
            let vol = _arguments["value"] as? Float
            MPVolumeView.setVolume(vol ?? 0.5)
            let val = AVAudioSession.sharedInstance().outputVolume
            result("Volume Updated: \(val)")
        case "volUp":
            let val = AVAudioSession.sharedInstance().outputVolume
            MPVolumeView.setVolume(val + 0.1)
            result("Volume Up")
        case "volDown":
            let val = AVAudioSession.sharedInstance().outputVolume
            MPVolumeView.setVolume(val - 0.1)
            result("Volume Down")
        case "orientLandscapeLeft":
            let val = UIInterfaceOrientation.landscapeLeft.rawValue
            UIDevice.current.setValue(val, forKey: "orientation")
            result("Orientation Changed to Landscape Left")
        case "orientLandscapeRight":
            let val = UIInterfaceOrientation.landscapeRight.rawValue
            UIDevice.current.setValue(val, forKey: "orientation")
            result("Orientation Changed to Landscape Right")
        case "orientLandscape":
            let val = UIInterfaceOrientation.landscapeLeft.rawValue
            UIDevice.current.setValue(val, forKey: "orientation")
            result("Orientation Changed to Landscape")
        case "orientPortrait":
            let val = UIInterfaceOrientation.portrait.rawValue
            UIDevice.current.setValue(val, forKey: "orientation")
            result("Orientation Changed to Portrait")
        case "orientPortraitUpsideDown":
            let val = UIInterfaceOrientation.portraitUpsideDown.rawValue
            UIDevice.current.setValue(val, forKey: "orientation")
            result("Orientation Changed to Portrait Upside Down")
        case "wifi":
            // Can not open wifi setting directly
            break
        case "checkWifi":
            result(checkNetworkStatus())
        case "bluetooth":
            // Can not open bluetooth setting directly
            break
        case "checkBluetooth":
            let utils = Utils()
            result(utils.checkBluetoothStatus()) // Not working
        default:
            result(nil)
        }
    }
    
    private func checkNetworkStatus() -> Bool {
        var zeroAddress = sockaddr_in()
        zeroAddress.sin_len = UInt8(MemoryLayout<sockaddr_in>.size)
        zeroAddress.sin_family = sa_family_t(AF_INET)
        
        guard let defaultRouteReachability = withUnsafePointer(to: &zeroAddress, {
            $0.withMemoryRebound(to: sockaddr.self, capacity: 1) {
                SCNetworkReachabilityCreateWithAddress(nil, $0)
            }
        }) else {
            return false
        }
        
        var flags: SCNetworkReachabilityFlags = []
        if !SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags) {
            return false
        }
        
        let isReachable = flags.contains(.reachable)
        let needsConnection = flags.contains(.connectionRequired)
        
        return (isReachable && !needsConnection)
    }
    
    private func sizeof <T> (_ : T.Type) -> Int
    {
        return (MemoryLayout<T>.size)
    }
    
    private func sizeof <T> (_ : T) -> Int
    {
        return (MemoryLayout<T>.size)
    }
    
    private func sizeof <T> (_ value : [T]) -> Int
    {
        return (MemoryLayout<T>.size * value.count)
    }
}

extension MPVolumeView {
    static func setVolume(_ volume: Float) {
        let volumeView = MPVolumeView()
        let slider = volumeView.subviews.first(where: { $0 is UISlider }) as? UISlider
        
        DispatchQueue.main.asyncAfter(deadline: DispatchTime.now() + 0.01) {
            slider?.value = volume
        }
    }
}

public class Utils: NSObject, CBCentralManagerDelegate {
    lazy var manager: CBCentralManager = { [unowned self] () -> CBCentralManager in
        CBCentralManager.init(delegate: self, queue: nil, options: [:])
    }()
    
    func centralManagerDidUpdateState(_ central: CBCentralManager!) {
        print("CentralManager is initialized")
        
        switch central.state {
        case .poweredOn:
            print("centralManagerDidUpdateState ::: Bluetooth is On.")
        case .poweredOff:
            print("centralManagerDidUpdateState ::: Bluetooth is Off.")
        case .resetting:
            print("centralManagerDidUpdateState ::: Bluetooth is Resetting.")
        case .unauthorized:
            print("centralManagerDidUpdateState ::: Bluetooth is Unauthorized.")
        case .unsupported:
            print("centralManagerDidUpdateState ::: Bluetooth is Unsupported.")
        case .unknown:
            print("centralManagerDidUpdateState ::: Bluetooth is unknown.")
        default:
            break
        }
    }
    
    func checkBluetoothStatus() -> Bool {
        switch manager.state {
        case .poweredOn:
            print("Bluetooth is On.")
            return true
        case .poweredOff:
            print("Bluetooth is Off.")
            return false
        case .resetting:
            print("Bluetooth is Resetting.")
            return false
        case .unauthorized:
            print("Bluetooth is Unauthorized.")
            return false
        case .unsupported:
            print("Bluetooth is Unsupported.")
            return false
        case .unknown:
            print("Bluetooth is unknown.")
            return false
        default:
            return false
        }
    }
}
